/*!=============================================================================
  ==============================================================================

  \file    SL_user.h

  \author  Stefan Schaal
  \date    March 2011

  ==============================================================================
  \remarks
  
  robot specific definitions for NAO Robot
  
  ============================================================================*/
  
#ifndef _SL_user_
#define _SL_user_

//! the robot name
#define ROBOT_NAME "nao"

// dimensions of the robot from Nao spec sheets
#define NeckOffsetZ	0.12650
#define ShoulderOffsetY	0.098
#define UpperArmLength	0.090
#define LowerArmLength	0.05055
#define ShoulderOffsetZ	0.100
#define HandOffsetX	0.058
#define HipOffsetZ	0.085
#define HipOffsetY	0.050
#define ThighLength	0.100
#define TibiaLength	0.10274
#define FootHeight	0.04511
#define HandOffsetZ	0.015
#define HandBumperX     0.090     //!< Length from wrist to hand bumper in x
#define HandBumperZ     0.015     //!< Offset of hand bumper in z
#define FingerOffsetX   0.070     //!< Length from wrist to first finger joint x
#define FingerOffsetZ   0.005     //!< Offset of first finger joint Z
#define FingerOffsetY   0.012     //!< Offset of finger joints in Y
#define ThumbOffsetX    0.05      //!< Length from wrist to thumb joint in x
#define ThumbOffsetZ    0.025     //!< Offset of thumb in z
#define SegmentLength   0.015     //!< Length of one finger segment
#define HeadLength      0.05      //!< position for head camera 

#define LFsrFLX         0.07025   //!< left foot front left foot sensor X
#define LFsrFLY         0.0399    //!< left foot front left foot sensor Y
#define LFsrFRX         0.07025   //!< left foot front right foot sensor X
#define LFsrFRY        -0.0331    //!< left foot front right foot sensor Y
#define LFsrRLX        -0.03025   //!< left foot rear left foot sensor X
#define LFsrRLY         0.0399    //!< left foot rear left foot sensor Y
#define LFsrRRX        -0.02965   //!< left foot rear right foot sensor X
#define LFsrRRY        -0.0291    //!< left foot rear right foot sensor Y

#define RFsrFLX         0.07025   //!< right foot front left foot sensor X
#define RFsrFLY         0.0331    //!< right foot front left foot sensor Y
#define RFsrFRX         0.07025   //!< right foot front right foot sensor X
#define RFsrFRY        -0.0399    //!< right foot front right foot sensor Y
#define RFsrRLX        -0.03025   //!< right foot rear left foot sensor X
#define RFsrRLY         0.0291    //!< right foot rear left foot sensor Y
#define RFsrRRX        -0.02965   //!< right foot rear right foot sensor X
#define RFsrRRY        -0.0399    //!< right foot rear right foot sensor Y


// links of the robot
enum RobotLinks {
  BASE,
	
  R_SHOULDER,
  R_ELBOW,
  R_WRIST,
	
  R_IF_BASE,
  R_IF_SEG1,
  R_IF_SEG2,
  R_IF_SEG3,
	
  R_MF_BASE,
  R_MF_SEG1,
  R_MF_SEG2,
  R_MF_SEG3,
	
  R_TH_BASE,
  R_TH_SEG1,
  R_TH_SEG2,
	
  R_HAND,
	
  L_SHOULDER,
  L_ELBOW,
  L_WRIST,
	
  L_IF_BASE,
  L_IF_SEG1,
  L_IF_SEG2,
  L_IF_SEG3,
	
  L_MF_BASE,
  L_MF_SEG1,
  L_MF_SEG2,
  L_MF_SEG3,
	
  L_TH_BASE,
  L_TH_SEG1,
  L_TH_SEG2,
	
  L_HAND,
	
  R_HIP,
  R_KNEE,
  R_ANKLE,
	
  R_FOOT,

  R_SR_FL,
  R_SR_FR,
  R_SR_RL,
  R_SR_RR,
	
  L_HIP,
  L_KNEE,
  L_ANKLE,
	
  L_FOOT,
	
  L_SR_FL,
  L_SR_FR,
  L_SR_RL,
  L_SR_RR,
	
  NECK,
	
  HEAD,
	
  N_ROBOT_LINKS
};

// endeffector information
enum RobotEndeffectors {
  RIGHT_HAND=1,
  LEFT_HAND,
  RIGHT_FOOT,
  LEFT_FOOT,

  N_ROBOT_ENDEFFECTORS
};

// vision variables
enum VisionCameras {
  CAMERA_1=1,
  CAMERA_2,

  N_VISION_CAMERAS
};

enum ColorBlobs {
  BLOB1=1,
  BLOB2,
  BLOB3,
  BLOB4,
  BLOB5,
  BLOB6,

  N_COLOR_BLOBS
};

// define the DOFs of this robot
enum RobotDOFs {
  R_SFE = 1,
  R_SAA,
  R_HR,
  R_EB,
  R_WR,
  R_FING,	
	
  L_SFE,
  L_SAA,
  L_HR,
  L_EB,
  L_WR,
  L_FING,
	
  R_FB,
  R_HFE,
  R_HAA,
  R_KFE,
  R_AFE,
  R_AAA,

  L_FB,	
  L_HFE,
  L_HAA,
  L_KFE,
  L_AFE,
  L_AAA,
	
  B_HR,
  B_HN,
	
  N_ROBOT_DOFS
};

enum NAORobotDOFs {
	
  RShoulderPitch = 1,
  RShoulderRoll,
  RElbowYaw,
  RElbowRoll,
  RWristYaw,
  RHand,	
	
  LShoulderPitch,
  LShoulderRoll,
  LElbowYaw,
  LElbowRoll,
  LWristYaw,
  LHand,
	
  RHipYawPitch,
  RHipPitch,
  RHipRoll,
  RKneePitch,
  RAnklePitch,
  RAnkleRoll,
	
  LHipYawPitch,
  LHipPitch,
  LHipRoll,
  LKneePitch,
  LAnklePitch,
  LAnkleRoll,
	
  HeadYaw,
  HeadPitch,
	
};

//! define miscellenous sensors of this robot
enum RobotMiscSensors {

  B_XDD_IMU = 1,
  B_YDD_IMU,
  B_ZDD_IMU,
  B_ADD_IMU,
  B_BDD_IMU,
  B_A_ANGLE,
  B_B_ANGLE,
  LF_COP_X,
  LF_COP_Y,
  LF_TOTAL_WEIGHT,
  RF_COP_X,
  RF_COP_Y,
  RF_TOTAL_WEIGHT,
  L_SONAR_VAL,
  R_SONAR_VAL,

  N_ROBOT_MISC_SENSORS
};


//! number of degrees-of-freedom of robot
#define N_DOFS (N_ROBOT_DOFS-1)

//! N_DOFS + fake DOFS, needed for parameter estimation; 
//   fake DOFS come from creating endeffector information
#define N_DOFS_EST (N_DOFS+33)

//! N_DOFS to be excluded from parameter estimation (e.g., eye joints);
//  these DOFS must be the last DOFS in the arrays
#define N_DOFS_EST_SKIP 0

//! number of links of the robot
#define N_LINKS    (N_ROBOT_LINKS-1)

//! number of miscellaneous sensors
#define N_MISC_SENSORS   (N_ROBOT_MISC_SENSORS-1)

//! number of endeffectors
#define N_ENDEFFS  (N_ROBOT_ENDEFFECTORS-1)

//! number of cameras used
#define N_CAMERAS (N_VISION_CAMERAS-1)

//! number of blobs that can be tracked by vision system
#define MAX_BLOBS (N_COLOR_BLOBS-1)

//! vision default post processing
#define VISION_DEFAULT_PP "vision_default.pp"

//! the servo rate used by the I/O with robot: this limits the
//  servo rates of all other servos 
#define  SERVO_BASE_RATE 100

//! divisor to obtain task servo rate (task servo can run slower than
//  base rate, but only in integer fractions */
#define  TASK_SERVO_RATIO   R1TO1

// settings for D/A debugging information -- see SL_oscilloscope.c 
#define   D2A_CM      1
#define   D2A_CT      2
#define   D2A_CV      3
#define   D2A_CR      4


#ifdef __cplusplus
extern "C" {
#endif

// function prototypes

  void
  compute_cog_jacobian(void);
  void
  compute_cog_velocity(void);
  void
  worldToBase(double *xw, SL_Cstate *cbase, SL_quat *obase, double *xl);
  void
  baseToWorld(double *xl, SL_Cstate *cbase, SL_quat *obase, double *xw);
  void
  compute_cog_kinematics(int **status, int flag, int use_z, int use_des, 
			 Matrix Jccogp, Matrix NJccog);
  void
  compute_constraint_cog_kinematics(Matrix NJc, int use_z, int **status, int use_des,
				    Matrix Jccog, 
				    Matrix Jccogp, Matrix NcJccog, 
				    Matrix NJccog);
  void
  compute_constraint_kinematics(int **status, int use_des, 
				Matrix Jc, Matrix Jcp, Matrix NJc);
  void
  worldToBaseVelocity(double *xw, double *vw, 
		      SL_Cstate *cbase, SL_quat *obase, double *vl);
  void 
  inverseDynamicsFloat(double dt, iMatrix status, int use_des, SL_DJstate *joint_ref_state,
		       double *xdd_ref, double *add_ref, Matrix fc);

// external variables

#ifdef __cplusplus
}
#endif


#endif  /* _SL_user_ */
