/*!=============================================================================
  ==============================================================================

  \file    SL_user_common.c

  \author 
  \date   

  ==============================================================================
  \remarks

      User file to declare common variables and functions shared by many 
      SL modules. This is a good place to declare global variables 

  ============================================================================*/

#include "SL.h"
#include "SL_user.h"
#include "SL_common.h"


/* global variables */
char joint_names[][20]= {
  {"BASE"},

  {"R_SFE"},
  {"R_SAA"},
  {"R_HR"},
  {"R_EB"},
  {"R_WR"},
  {"R_FING"},	
	
  {"L_SFE"},
  {"L_SAA"},
  {"L_HR"},
  {"L_EB"},
  {"L_WR"},
  {"L_FING"},
	
  {"R_FB"},
  {"R_HFE"},
  {"R_HAA"},
  {"R_KFE"},
  {"R_AFE"},
  {"R_AAA"},

  {"L_FB"},	
  {"L_HFE"},
  {"L_HAA"},
  {"L_KFE"},
  {"L_AFE"},
  {"L_AAA"},
	
  {"B_HR"},
  {"B_HN"}

};

char cart_names[][20]= {
  {"dummy"},
  {"RIGHT_HAND"},
  {"LEFT_HAND"},
  {"RIGHT_FOOT"},
  {"LEFT_FOOT"}
};

char blob_names[][20]= {
  {"dummy"},
  {"BLOB1"},
  {"BLOB2"},
  {"BLOB3"},
  {"BLOB4"},
  {"BLOB5"},
  {"BLOB6"}
};

char link_names[][20]= {
  {"BASE"},

  {"R_SHOULDER"},
  {"R_ELBOW"},
  {"R_WRIST"},
	
  {"R_IF_BASE"},
  {"R_IF_SEG1"},
  {"R_IF_SEG2"},
  {"R_IF_SEG3"},
	
  {"R_MF_BASE"},
  {"R_MF_SEG1"},
  {"R_MF_SEG2"},
  {"R_MF_SEG3"},
	
  {"R_TH_BASE"},
  {"R_TH_SEG1"},
  {"R_TH_SEG2"},
	
  {"R_HAND"},
	
  {"L_SHOULDER"},
  {"L_ELBOW"},
  {"L_WRIST"},
	
  {"L_IF_BASE"},
  {"L_IF_SEG1"},
  {"L_IF_SEG2"},
  {"L_IF_SEG3"},
	
  {"L_MF_BASE"},
  {"L_MF_SEG1"},
  {"L_MF_SEG2"},
  {"L_MF_SEG3"},
	
  {"L_TH_BASE"},
  {"L_TH_SEG1"},
  {"L_TH_SEG2"},
	
  {"L_HAND"},
	
  {"R_HIP"},
  {"R_KNEE"},
  {"R_ANKLE"},
	
  {"R_FOOT"},

  {"R_SR_FL"},
  {"R_SR_FR"},
  {"R_SR_RL"},
  {"R_SR_RR"},
	
  {"L_HIP"},
  {"L_KNEE"},
  {"L_ANKLE"},
	
  {"L_FOOT"},
	
  {"L_SR_FL"},
  {"L_SR_FR"},
  {"L_SR_RL"},
  {"L_SR_RR"},
	
  {"NECK"},
	
  {"HEAD"}

};

char misc_sensor_names[][20]= {
  {"dummy"},
  {"B_XDD_IMU"},
  {"B_YDD_IMU"},
  {"B_ZDD_IMU"},
  {"B_ADD_IMU"},
  {"B_BDD_IMU"},
  {"B_A_ANGLE"},
  {"B_B_ANGLE"},
  {"LF_COP_X"},
  {"LF_COP_Y"},
  {"LF_TOTAL_WEIGHT"},
  {"RF_COP_X"},
  {"RF_COP_Y"},
  {"RF_TOTAL_WEIGHT"},
  {"L_SONAR_VAL"},
  {"R_SONAR_VAL"}
};

/* initialization needs to be done for this mapping */
int           link2endeffmap[] = {0,R_HAND,L_HAND,R_FOOT,L_FOOT};


/* the following include must be the last line of the variable declaration section */
#include "SL_user_common.h"   /* do not erase!!! */



/*!*****************************************************************************
 *******************************************************************************
\note  setDefaultEndeffector
\date  June 1999
   
\remarks 

        assigns default values to the endeffector parameters

 *******************************************************************************
 Function Parameters: [in]=input,[out]=output

     none

 ******************************************************************************/
void 
setDefaultEndeffector(void) {

  int i;

  for (i=1; i<=N_ENDEFFS; ++i) {
    endeff[i].m       = 0.0;
    endeff[i].mcm[_X_]= 0.0;
    endeff[i].mcm[_Y_]= 0.0;
    endeff[i].mcm[_Z_]= 0.0;
    endeff[i].x[_X_]  = 0.0;
    endeff[i].x[_Y_]  = 0.0;
    endeff[i].x[_Z_]  = 0.0;
    endeff[i].a[_A_]  = 0.0;
    endeff[i].a[_B_]  = 0.0;
    endeff[i].a[_G_]  = -PI/2.;
  }

  // special parameters
  endeff[RIGHT_HAND].x[_X_]  = HandBumperX;
  endeff[RIGHT_HAND].x[_Z_]  = HandBumperZ;
  endeff[LEFT_HAND].x[_X_]   = HandBumperX;
  endeff[LEFT_HAND].x[_Z_]   = HandBumperZ;
  endeff[RIGHT_FOOT].x[_Z_]  = -FootHeight;
  endeff[LEFT_FOOT].x[_Z_]   = -FootHeight;

}


/*!*****************************************************************************
 *******************************************************************************
\note  setRealRobotOptions
\date  June 1999
   
\remarks 

        allows parameter changes if SL runs for a real robot instead
        of a simulation

 *******************************************************************************
 Function Parameters: [in]=input,[out]=output

     none

 ******************************************************************************/
void
setRealRobotOptions(void) 
{

  if (!real_robot_flag)
    sprintf(config_files[CONFIGFILES],"ConfigFilesSim.cf");
  else
    sprintf(config_files[CONFIGFILES],"ConfigFiles.cf");

  // update the config file names
  read_config_files(config_files[CONFIGFILES]);
  
  return;

}

