/*!=============================================================================
  ==============================================================================

  \file    SL_user_simulation.c

  \author  Stefan Schaal
  \date    Nov. 2007

  ==============================================================================
  \remarks
  
  This program contains all user functions for the numerical simluation of
  a robot
  
  ============================================================================*/
  
// SL general includes of system headers
#include "SL_system_headers.h"

/* user specific headers */
#include "SL.h"
#include "SL_integrate.h"
#include "SL_user.h"
#include "SL_common.h"
#include "SL_objects.h"
#include "SL_simulation_servo.h"
#include "utility.h"
#include "utility_macros.h"
#include "mdefs.h"
#include "SL_dynamics.h"
#include "SL_kinematics.h"

/* global variables */

/* local variables */
  
/* local functions */

/* external functions */


/*!*****************************************************************************
 *******************************************************************************
 \note  init_user_simulation
 \date  July 1998
 
 \remarks 
 
 initializes everything needed for the numerical simulation.
 
 *******************************************************************************
 Function Parameters: [in]=input,[out]=output

 none 

 ******************************************************************************/
int
init_user_simulation(void)

{
  
  int i,j,n;

  // initalize objects in the environment
  readObjects(config_files[OBJECTS]);

  // assign contact force mappings
#include "LEKin_contact.h"

  // change numerical integration
  n_integration = 80;
  integrate_method = INTEGRATE_EULER;

  // change the freeze_base_pos[] array if needed

  // zero the state of the robot
  reset();
  

  return TRUE;
}


/*!*****************************************************************************
 *******************************************************************************
\note  run_user_simulation
\date  Nov. 7, 2007
   
\remarks 

      computes additional simulation values, e.g., misc sensors

 *******************************************************************************
 Function Parameters: [in]=input,[out]=output


 ******************************************************************************/
int
run_user_simulation(void)

{
  int    i,j;
  double mass;

  double    xcop[N_CART+1];
  double    x[N_CART+1];
  double    sum;

  MY_MATRIX(Xmcog,0,N_DOFS,1,3);
  MY_MATRIX(Xaxis,0,N_DOFS,1,3);
  MY_MATRIX(Xorigin,0,N_DOFS,1,3);
  MY_MATRIX(Xlink,0,N_LINKS,1,3);
  MY_MATRIX_ARRAY(Ahmat,1,4,1,4,N_LINKS);
  MY_MATRIX_ARRAY(Ahmatdof,1,4,1,4,N_DOFS);


  // compute the link information for this state
  linkInformation(joint_sim_state, &base_state, &base_orient,
		  endeff, Xmcog, Xaxis, Xorigin, Xlink,
		  Ahmat, Ahmatdof);


  for (j=1; j<=N_CART; ++j) {
    cog.x[j]     = Xmcog[BASE][j];
  }

  mass = links[BASE].m;

  for (i=1; i<=n_dofs; ++i) {
    for (j=1; j<=N_CART; ++j) {
      cog.x[j]     += Xmcog[i][j];
    }
    mass += links[i].m;
  }

  if (mass != 0 ){
    for (j=1; j<=N_CART; ++j) {
      cog.x[j] /= mass;
    }
  } else {
    for (j=1; j<=N_CART; ++j) {
      cog.x[j] = 0;
    }
  }

  //printf("COG: %5.3f %5.3f %5.3f  %5.3f\n",cog.x[1],cog.x[2],cog.x[3],mass*G);



  for (j=1; j<=N_CART; ++j)
    xcop[j]=0;
  sum = 1.e-10;

  for (i=1; i<=n_contacts; ++i) { /* loop over all foot contact points */

    // check whether there is an active contact
    if (!contacts[i].active || !contacts[i].status)
      continue;


      //printf("contact = %d\n",i);

      // where is this contact point
      computeContactPoint(&(contacts[i]),link_pos_sim,Alink_sim,x);

      // accumulate the cop position as a weighted sum, where the weight is the
      // z component of the force at each contact point
      for (j=1; j<=N_CART; ++j)
	xcop[j] += x[j] * contacts[i].f[_Z_];
      
      sum += contacts[i].f[_Z_];



  }

  for (j=1; j<=N_CART; ++j)
    xcop[j] /= sum;

  //printf("COP: %5.3f %5.3f %5.3f  %5.3f\n\n",xcop[1],xcop[2],xcop[3],sum);



  return TRUE;
}


